package ink.wgink.gateway.handler.count;

import ink.wgink.gateway.dao.log.IRequestLogDao;
import ink.wgink.gateway.dao.route.IRouteDao;
import ink.wgink.gateway.dao.routetype.IRouteTypeDao;
import ink.wgink.gateway.handler.BaseHandler;
import ink.wgink.gateway.pojo.count.HomeCount;
import ink.wgink.gateway.pojo.count.RequestLogCount;
import ink.wgink.gateway.pojo.log.RequestLog;
import ink.wgink.gateway.pojo.result.SuccessResult;
import ink.wgink.gateway.pojo.result.SuccessResultData;
import ink.wgink.gateway.util.DateUtil;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.ExampleMatcher;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;
import org.springframework.web.reactive.function.server.ServerRequest;
import org.springframework.web.reactive.function.server.ServerResponse;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.time.Duration;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: CountHandler
 * @Description: 统计
 * @Author: wanggeng
 * @Date: 2021/5/15 3:01 下午
 * @Version: 1.0
 */
@Service
public class CountHandler extends BaseHandler {

    private IRouteTypeDao routeTypeDao;
    private IRouteDao routeDao;
    private IRequestLogDao requestLogDao;

    public CountHandler(IRouteTypeDao routeTypeDao, IRouteDao routeDao, IRequestLogDao requestLogDao) {
        this.routeTypeDao = routeTypeDao;
        this.routeDao = routeDao;
        this.requestLogDao = requestLogDao;
    }

    /**
     * 所有路由类型
     *
     * @param serverResponse
     * @return
     */
    public Mono<ServerResponse> countHome(ServerRequest serverResponse) {
        return ServerResponse.ok().contentType(MediaType.TEXT_EVENT_STREAM).body(
                Flux.interval(Duration.ofSeconds(0), Duration.ofSeconds(15))
                        .flatMap(second -> routeTypeDao.count()
                                .flatMap(routeTypeCount -> routeDao.count()
                                        .flatMap(routeCount -> requestLogDao.count()
                                                .flatMap(requestCount -> {
                                                    RequestLog requestLogExample = new RequestLog();
                                                    requestLogExample.setGmtCreate(DateUtil.getDay());

                                                    ExampleMatcher exampleMatcher = ExampleMatcher.matching()
                                                            .withMatcher("gmtCreate", ExampleMatcher.GenericPropertyMatcher::contains)
                                                            .withIgnoreCase("id");
                                                    Example example = Example.of(requestLogExample, exampleMatcher);

                                                    return requestLogDao.count(example)
                                                            .flatMap(requestTodayCount -> {
                                                                HomeCount homeCount = new HomeCount();
                                                                homeCount.setRouteTypeCount(routeTypeCount);
                                                                homeCount.setRouteCount(routeCount);
                                                                homeCount.setRequestCount(requestCount);
                                                                homeCount.setRequestTodayCount((Long) requestTodayCount);
                                                                return Mono.just(homeCount);
                                                            });
                                                })
                                        )
                                )
                        ),
                HomeCount.class);
    }

    /**
     * 最新请求数量列表
     *
     * @param serverRequest
     * @return
     */
    // 每分钟跟新依次统计数据
    public Mono<ServerResponse> listLastCount(ServerRequest serverRequest) {
        return ServerResponse.ok().contentType(MediaType.TEXT_EVENT_STREAM).body(requestLogDao.count().flatMap(count -> {
            RequestLogCount requestLogCount = new RequestLogCount();
            return Mono.just(requestLogCount);
        }), RequestLogCount.class);
    }

}
